-- This file assumes that you have executed 05_certsignserver.sql.
SET NOCOUNT ON
USE PermTest
go
-- We opened a support case, and CSS saw that we were using our own
-- system procedure. They made it clear that this is very unsupported,
-- so we will move the procedure to the user databases.

go
-- Here is a database-local version of sp_showusersindb to be more
-- supported.
CREATE PROCEDURE ShowSessions AS
   SELECT name, type, usage FROM sys.login_token

   IF is_srvrolemember('PowerUsers') = 1 OR
      has_perms_by_name(NULL, NULL, 'CONTROL SERVER') = 1
     SELECT * FROM sys.dm_exec_sessions WHERE database_id = db_id()
   ELSE
     RAISERROR('You are not permitted to run this procedure', 16, 1)
go

-- Test with the power user - only sees its own session.
EXECUTE AS LOGIN = 'PermTestAdmin'
go
EXECUTE ShowSessions
go
REVERT


-- Move to master.
USE master

-- To be able to sign a procedure in the PermTest database with the
-- DemoServerCert, we need to export it from master to PermTest.
-- From SQL 2012 and up, we can use metadata functions to get the
-- public and private keys to copy them to PermTest.

-- Here is the concept:
DECLARE @public_key varbinary(MAX) =
            certencoded(cert_id('DemoServerCert')),
        -- When retrieving the private key we must give the password
        -- for it. And specify a password to protect in the variable.
        -- They could be different if you wanted to.
        @private_key varbinary(MAX) =
            certprivatekey(cert_id('DemoServerCert'),
                           'While My Guitar Gently Weeps',
                           'While My Guitar Gently Weeps')
-- Change database.
USE PermTest
-- Create the certificate in this database, using the FROM BINARY option.
-- Alas, this syntax is not valid - the binary values must be literals.
CREATE CERTIFICATE DemoServerCert
FROM BINARY = @public_key
WITH PRIVATE KEY
   (BINARY = @private_key,
    DECRYPTION BY PASSWORD = 'While My Guitar Gently Weeps',
    ENCRYPTION BY PASSWORD = 'While My Guitar Gently Weeps')
go


-- We need to use dynamic SQL, sigh.
USE master
go
DECLARE @public_key varbinary(MAX) =
            certencoded(cert_id('DemoServerCert')),
        -- Again, we must give passwords to retrieve and to protect.
        @private_key varbinary(MAX) =
            certprivatekey(cert_id('DemoServerCert'),
                           'While My Guitar Gently Weeps',
                           'While My Guitar Gently Weeps'),
        @sql nvarchar(MAX)

-- A debug SELECT, so if the SQL comes out blank, you can see which
-- key you did not retrieve.
--SELECT @public_key, @private_key

-- Code to create the certificates from the bytes we extracted.
SELECT @sql =
   'CREATE CERTIFICATE DemoServerCert
    FROM BINARY = ' + convert(varchar(MAX), @public_key, 1) + '
    WITH PRIVATE KEY
        (BINARY = ' + convert(varchar(MAX), @private_key, 1) + ',
         DECRYPTION BY PASSWORD = ''While My Guitar Gently Weeps'',
         ENCRYPTION BY PASSWORD = ''While My Guitar Gently Weeps'')'

PRINT  @sql
-- Execute the batch in PermTest
EXEC PermTest..sp_executesql @sql
go

-- Move to PermTest again
USE PermTest
go
-- Sign the procedure.
ADD SIGNATURE TO ShowSessions BY CERTIFICATE DemoServerCert
                 WITH PASSWORD = 'While My Guitar Gently Weeps'
go
-- Drop the private key. This makes sure that PermTestAdmin cannot use the
-- certificate, even if he/she knows the password.
ALTER CERTIFICATE DemoServerCert REMOVE PRIVATE KEY
go
-- The power user can now run the procedure.
EXECUTE AS LOGIN = 'PermTestAdmin'
go
EXECUTE ShowSessions
go
REVERT

------------------------------------------------------------------------------------
-- On SQL 2005 and SQL 2008, you cannot use the above, but you can
-- use BACKUP CERTIFICATE which bounces the certificate over disk.

-- Make sure that the signature and the certificate is not in PermTest,
-- in case you ran the above.
USE PermTest
go
DROP SIGNATURE FROM ShowSessions BY CERTIFICATE DemoServerCert
go
DROP CERTIFICATE DemoServerCert

USE master
go
-- Backup the certificate to disk. For the private key we need to give the password
-- to retrieve the key. And we need to give a password to protect it on
-- disk - that could be a different password.
BACKUP CERTIFICATE DemoServerCert
    TO FILE='C:\temp\DemoServerCert.cer'
WITH PRIVATE KEY
   (FILE = 'C:\temp\DemoServerCert.pvk',
    DECRYPTION BY PASSWORD = 'While My Guitar Gently Weeps',
    ENCRYPTION BY PASSWORD = 'While My Guitar Gently Weeps')

-- Move over to the PermTest database.
USE PermTest
go
-- Import the certificate from the files. Again, we must give the
-- password that protects it in the file, and a password to protect it
-- in this database.
CREATE CERTIFICATE DemoServerCert
    FROM FILE='C:\temp\DemoServerCert.cer'
WITH PRIVATE KEY
    (FILE = 'C:\temp\DemoServerCert.pvk',
     DECRYPTION BY PASSWORD = 'While My Guitar Gently Weeps',
     ENCRYPTION BY PASSWORD = 'While My Guitar Gently Weeps')

-- Delete the certificate from disk, so that next export does not fail.
EXEC xp_cmdshell 'DEL C:\temp\DemoServerCert.*', 'no_output'
go


-- Sign procedure and drop private key.
ADD SIGNATURE TO ShowSessions BY CERTIFICATE DemoServerCert
    WITH PASSWORD = 'While My Guitar Gently Weeps'
ALTER CERTIFICATE DemoServerCert REMOVE PRIVATE KEY
go

-- Play our power user.
EXECUTE AS LOGIN = 'PermTestAdmin'
go
EXECUTE ShowSessions
go
REVERT



